<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" xmlns="urn:WebWorks-Document-Schema" 
                              xmlns:wwdoc="urn:WebWorks-Document-Schema" 
                              xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
>
 <xsl:output method="xml" encoding="utf-8" indent="yes" />
 <xsl:namespace-alias stylesheet-prefix="wwdoc" result-prefix="#default" />
 <xsl:strip-space elements="*" />
 
 <xsl:template match="/">
  <xsl:param name="ParamContext" select="." />
  
  <wwdoc:Document version="1.1">
   <wwdoc:Styles>
    <wwdoc:ParagraphStyles>
     <wwdoc:ParagraphStyle name="db-section-1" />
     <wwdoc:ParagraphStyle name="db-section-2" />
     <wwdoc:ParagraphStyle name="db-section-3" />
     <wwdoc:ParagraphStyle name="db-section-4" />
     <wwdoc:ParagraphStyle name="db-section-5" />
     <wwdoc:ParagraphStyle name="db-para" />
     <wwdoc:ParagraphStyle name="db-para-indent-1" />
     <wwdoc:ParagraphStyle name="db-para-indent-2" />
     <wwdoc:ParagraphStyle name="db-para-indent-3" />
     <wwdoc:ParagraphStyle name="db-para-numbered-1" />
     <wwdoc:ParagraphStyle name="db-para-numbered-2" />
     <wwdoc:ParagraphStyle name="db-para-bulleted-1" />
     <wwdoc:ParagraphStyle name="db-para-bulleted-2" />
     <wwdoc:ParagraphStyle name="db-para-bulleted-3" />
     <wwdoc:ParagraphStyle name="db-para-cell-head" />
     <wwdoc:ParagraphStyle name="db-para-cell-body" />
     <wwdoc:ParagraphStyle name="db-para-cell-foot" />
     <wwdoc:ParagraphStyle name="db-screen" />
    </wwdoc:ParagraphStyles>
    
    <wwdoc:CharacterStyles>
     <wwdoc:CharacterStyle name="db-emphasis-strong" />
    </wwdoc:CharacterStyles>
    
    <wwdoc:TableStyles>
     <wwdoc:TableStyle name="docbook1" />
    </wwdoc:TableStyles>
   </wwdoc:Styles>
   <wwdoc:Content>
    <xsl:apply-templates select="$ParamContext/article" />
   </wwdoc:Content>
  </wwdoc:Document>
 </xsl:template>
 
 <xsl:template match="article">
  <xsl:param name="ParamContext" select="." />

  <xsl:apply-templates select="$ParamContext/section" />
 </xsl:template>
 
 <xsl:template match="section">
  <xsl:param  name="ParamContext" select="." />

  <xsl:variable name="VarId">
   <xsl:text>p</xsl:text>
   <xsl:number level="multiple" count="section" format="a.a" />
  </xsl:variable>
  
  <xsl:variable name="VarStyleName">
   <xsl:text>db-section-</xsl:text>
   <xsl:value-of select="count(ancestor-or-self::section)" />
  </xsl:variable>

  <wwdoc:Paragraph id="{$VarId}" stylename="{$VarStyleName}">
   <xsl:apply-templates select="$ParamContext/title" mode="section" />
  </wwdoc:Paragraph>

  <xsl:apply-templates select="$ParamContext/*[local-name() != 'section']" />
  <xsl:apply-templates select="$ParamContext/section" />
 </xsl:template>

 <xsl:template match="title">
  <!-- no op -->
 </xsl:template>

 <xsl:template match="title" mode="section">
  <xsl:param name="ParamContext" select="." />

  <wwdoc:Number value="{substring-before($ParamContext/generated/text(), '&#xA0;')}">
   <wwdoc:Text value="{normalize-space($ParamContext/generated/text())}" />
  </wwdoc:Number>
  <wwdoc:TextRun>
   <wwdoc:Text value="{$ParamContext/text()}" />
  </wwdoc:TextRun>
 </xsl:template>
 
 <xsl:template match="para">
  <xsl:param name="ParamContext" select="." />

  <xsl:variable name="VarId">
   <xsl:number level="multiple" count="*" format="a.a" />
  </xsl:variable>

  <wwdoc:Paragraph id="{$VarId}" stylename="db-para">
   <xsl:for-each select="text() | node()[string-length(local-name()) &gt; 0]">
    <xsl:apply-templates select="." mode="character" />
   </xsl:for-each>
  </wwdoc:Paragraph>
 </xsl:template>

 <xsl:template match="itemizedlist | orderedlist">
  <xsl:param name="ParamContext" select="." />

  <xsl:apply-templates select="$ParamContext/listitem" />
 </xsl:template>

 <xsl:template match="listitem">
  <xsl:param name="ParamContext" select="." />

  <xsl:variable name="VarId">
   <xsl:number level="multiple" count="*" format="a.a" />
  </xsl:variable>

  <xsl:variable name="VarLevel" select="count($ParamContext/ancestor::itemizedlist | $ParamContext/ancestor::orderedlist)" />

  <xsl:variable name="VarParentName" select="local-name($ParamContext/parent::node()[1])" />

  <xsl:variable name="VarType">
   <xsl:choose>
    <xsl:when test="$VarParentName = 'itemizedlist'">
     <xsl:text>bulleted</xsl:text>
    </xsl:when>

    <xsl:when test="$VarParentName = 'orderedlist'">
     <xsl:text>numbered</xsl:text>
    </xsl:when>
   </xsl:choose>
  </xsl:variable>
  
  <xsl:variable name="VarStyleName">
   <xsl:text>db-para-</xsl:text>
   <xsl:value-of select="$VarType" />
   <xsl:text>-</xsl:text>
   <xsl:value-of select="$VarLevel" />
  </xsl:variable>

  <xsl:variable name="VarNumber">
   <xsl:choose>
    <xsl:when test="$VarParentName = 'itemizedlist'">
     <xsl:value-of select="'&#x2022;'" />
     <xsl:text>&#xA0;</xsl:text>
    </xsl:when>

    <xsl:when test="$VarParentName = 'orderedlist'">
     <xsl:number level="single" format="1" />
     <xsl:text>.&#xA0;</xsl:text>
    </xsl:when>
   </xsl:choose>
  </xsl:variable>

  <wwdoc:Paragraph id="{$VarId}" stylename="{$VarStyleName}">
   <wwdoc:Number value="{$VarNumber}">
    <wwdoc:Text value="{$VarNumber}" />
   </wwdoc:Number>

   <xsl:for-each select="text() | node()[string-length(local-name()) &gt; 0 and local-name() != 'itemizedlist']">
    <xsl:apply-templates select="." mode="character" />
   </xsl:for-each>
  </wwdoc:Paragraph>
  
  <xsl:apply-templates select="itemizedlist | orderedlist" />
 </xsl:template>
 
 <xsl:template match="screen">
  <xsl:param name="ParamContext" select="." />

  <xsl:variable name="VarId">
   <xsl:number level="multiple" count="*" format="a.a" />
  </xsl:variable>

  <wwdoc:Paragraph id="{$VarId}" stylename="db-screen">
   <xsl:apply-templates select="text()" mode="preserve-space" />
  </wwdoc:Paragraph>
 </xsl:template>

 <xsl:template match="block_quote">
  <xsl:param name="ParamContext" select="." />
  
  <xsl:apply-templates select="$ParamContext/*" />
 </xsl:template>

 <xsl:template match="table">
  <xsl:param name="ParamContext" select="." />
  
  <xsl:variable name="VarId">
   <xsl:text>t</xsl:text>
   <xsl:number level="multiple" count="*" format="a.a" />
  </xsl:variable>

  <wwdoc:Table id="{$VarId}" stylename="docbook1">
   <wwdoc:Style>
    <wwdoc:Attribute name="text-align" value="left" />
    <wwdoc:TableHead>
     <wwdoc:Style />
    </wwdoc:TableHead>
    <wwdoc:TableBody>
     <wwdoc:Style />
    </wwdoc:TableBody>
    <wwdoc:TableFoot>
     <wwdoc:Style />
    </wwdoc:TableFoot>
   </wwdoc:Style>
 
   <xsl:apply-templates select="$ParamContext/*" />
  </wwdoc:Table>
 </xsl:template>
 
 <xsl:template match="tgroup">
  <xsl:param name="ParamContext" select="." />
  
  <wwdoc:TableColumns value="{$ParamContext/@cols}">
   <xsl:for-each select="$ParamContext/colspec">
    <wwdoc:TableColumn width="{@colwidth}" />
   </xsl:for-each>
  </wwdoc:TableColumns>
  
  <wwdoc:Style>
   <wwdoc:Attribute name="width" value="" />
  </wwdoc:Style>
  
  <wwdoc:Caption />
  
  <xsl:apply-templates select="$ParamContext/thead | $ParamContext/tbody | $ParamContext/tfoot" />
 </xsl:template>
 
 <xsl:template match="thead">
  <xsl:param name="ParamContext" select="." />
  
  <wwdoc:TableHead>
   <xsl:apply-templates select="$ParamContext/row" />
  </wwdoc:TableHead>
 </xsl:template>
 
 <xsl:template match="tbody">
  <xsl:param name="ParamContext" select="." />
  
  <wwdoc:TableBody>
   <xsl:apply-templates select="$ParamContext/row" />
  </wwdoc:TableBody>
 </xsl:template>
 
 <xsl:template match="tfoot">
  <xsl:param name="ParamContext" select="." />
  
  <wwdoc:TableFoot>
   <xsl:apply-templates select="$ParamContext/row" />
  </wwdoc:TableFoot>
 </xsl:template>
 
 <xsl:template match="row">
  <xsl:param name="ParamContext" select="." />
  <xsl:param name="ParamPosition" select="position()" />

  <wwdoc:TableRow>
   <xsl:apply-templates select="$ParamContext/entry">
    <xsl:with-param name="ParamRowPosition" select="$ParamPosition" />
   </xsl:apply-templates>
  </wwdoc:TableRow>
 </xsl:template>
 
 <xsl:template match="entry">
  <xsl:param name="ParamContext" select="." />
  <xsl:param name="ParamRowPosition" select="1" />

  <xsl:variable name="VarBackgroundColor">
   <xsl:choose>
    <xsl:when test="count(ancestor::thead[1]) = 1">
     <xsl:value-of select="'lightgreen'" />
    </xsl:when>

    <xsl:when test="($ParamRowPosition mod 2) = 0">
     <xsl:value-of select="'whitesmoke'" />
    </xsl:when>
   </xsl:choose>
  </xsl:variable>

  <wwdoc:TableCell>
   <xsl:if test="string-length($VarBackgroundColor) &gt; 0">
    <wwdoc:Style>
     <wwdoc:Attribute name="background-color" value="{$VarBackgroundColor}" />
    </wwdoc:Style>
   </xsl:if>

   <xsl:apply-templates select="$ParamContext/*" />
  </wwdoc:TableCell>
 </xsl:template>

 <xsl:template match="itemizedlist" mode="character">
  <xsl:param name="ParamContext" select="." />

  <xsl:apply-templates select="$ParamContext" />
 </xsl:template>

 <xsl:template match="node()" mode="character">
  <xsl:param name="ParamContext" select="." />

  <xsl:variable name="VarId">
   <xsl:text>tr-</xsl:text>
   <xsl:number level="multiple" count="*" format="a.a" />
   <xsl:text>.</xsl:text>
   <xsl:value-of select="position()" />
  </xsl:variable>

  <wwdoc:TextRun id="{$VarId}">
   <xsl:if test="string-length(local-name($ParamContext)) &gt; 0">
    <xsl:attribute name="stylename">
     <xsl:text>db-</xsl:text>
     <xsl:value-of select="local-name($ParamContext)" />
     <xsl:if test="string-length($ParamContext/@role) &gt; 0">
      <xsl:text>-</xsl:text>
      <xsl:value-of select="$ParamContext/@role" />
     </xsl:if>
    </xsl:attribute>
   </xsl:if>
   <wwdoc:Text value="{$ParamContext | $ParamContext/text()}" />
  </wwdoc:TextRun>
 </xsl:template>
 
 <xsl:template name="Preserve-Space" match="node()" mode="preserve-space">
  <xsl:param name="ParamContext" select="." />
  
  <xsl:variable name="VarId">
   <xsl:text>tr-</xsl:text>
   <xsl:number level="multiple" count="*" format="a.a" />
  </xsl:variable>
  
  <xsl:choose>
  <xsl:when test="contains($ParamContext, '&#xD;&#xA;')">
   <wwdoc:TextRun id="{$VarId}">
    <wwdoc:Text value="{substring-before($ParamContext, '&#xD;&#xA;')}" />
   </wwdoc:TextRun>
   <wwdoc:LineBreak />
   
   <xsl:call-template name="Preserve-Space">
    <xsl:with-param name="ParamContext" select="substring-after($ParamContext, '&#xD;&#xA;')" />
   </xsl:call-template>
  </xsl:when>

  <xsl:otherwise>
   <wwdoc:TextRun id="{$VarId}">
    <wwdoc:Text value="{$ParamContext}" />
   </wwdoc:TextRun>
  </xsl:otherwise>
  </xsl:choose>
 </xsl:template>
</xsl:stylesheet>